*+
*+
*+    Source Module => D:\INCOMING\TIMEOUT.PRG
*+
*+    Copyright (C) 1997, Phil Barnett. All Rights Reserved Worldwide.
*+
*+    Functions: Procedure CHECKINACTIVE()
*+               Function Main()
*+               Procedure CHECKLOGOUT()
*+               Procedure AppsQuit()
*+               Function datadir()
*+               Function TimeAsString()
*+
*+    Reformatted by Click! 1.13b3 on May-25-1998 at  2:18 pm
*+
*+

/*
Dear Rico,

Rico Suave wrote:
> 
> My users are not exiting the program at the end of the day. The DBF's
> are opened, therefore can't be backed up to tape at night.
> 
> How can I automatically quit the app after, say, 2hrs of inactivity,
> or after 10pm?

Use background events!
  http://www.stack.nl/~lamping/bgevents.htm

DEFINE BGEVENT ACTION APPSQUIT() WHEN TIME() > "22:00:00" INTERVAL 5*60 
// check every five minutes

Checking inactivity can only be done by registering the moment that a
mouse(key) was hit. You have to use a function myinkey() and store the
time. Use dseconds() in bgevents.ch (in zipfile on my website). It
returns numbers of seconds since the beginning of the century, to avoid
the midnight syndrome. You should store the time in a public variable or
using a getsetfunction which uses a static variable to hold a value.
Lets say you use a public variable named nLASTHIT

With this value you can check inactivaty using a background event.

*/

DEFINE BGEVENT ACTION CHECKINACTIVE( 2 * 60 * 60 ) INTERVAL 5 * 60              // every
five minutes

*+
*+
*+    Procedure CHECKINACTIVE()
*+
*+    Called from ( timeout.prg  )   1 - 
*+
*+
*+
procedure CHECKINACTIVE( nSeconds )

if ( dSeconds() - nLasthit ) > nSeconds
   Appsquit()
endif

return

/*
These are the answers to your questions.

An easier way and more flexible way is to force logout of all user upon
demand. (idea of Philippe Morchain, Paris)
The following source has been tested and works!
You place a file logout.now in a predefined directory. The applications
checks for this file and as soon as it has been
spotted it will notify the user that he/she has to finish the current
job and leave the application. If not the application itself will do so.

Let me know if this works for you.

If you wish to use FT_ONIDLE() instead of the BGEVENTS event-handler,
then you can use

  BGE_IDLEFUNC( {|b| ft_ONIDLE(b) } )

The events are then only handled if Clipper is idle. The original
event-handler links into the maindos-loop and is evaluated continuously 
even when not idle.

Good luck and write back!

Robert Lamping
Creator of Background events.
*/

// Here is the source:

/*
>From    : Robert Franklin Lamping
Date    : Januari 10, 1998
Subject : Sample program to force logout in a network using
          background events.
Internet: http://www.stack.nl/~lamping/bgevents.htm
Email   : Robert@winning.com

Compile and link:
*----------------
@echo off
Clipper testme     /N /W /A /M

REM Compile bgevents for your Clipper version:
clipper bgevents

blinker fi %1, BGEVENTS, EVENTS
*/

#include "bgevents.ch"

*+
*+
*+    Function Main()
*+
*+
*+
function Main

// BGE_IDLEFUNC( {|b| ft_ONIDLE(b) } )
BACKGROUND EVENTS INTERVAL .2

// DEFINE BGEVENT ACTION FT_IAMIDLE(50, .T.) // give some timeslices away
// * Look for latest FT_IAMIDLE() in file NFPAT1a.zip
DEFINE BGEVENT ACTION( devpos( 0, 0 ), devout( time() ) )   // show time
DEFINE BGEVENT ACTION CHECKLOGOUT() NAME CHECKLOGOUT INTERVAL 5
DEFINE BGEVENT ACTION APPSQUIT() NAME APPSQUIT INTERVAL 5 * 60 INACTIVE         // and quit

//  Keep EVENT APPSQUIT Inactive!
//  it is mentioned here so that you can change the interval value at
//  a central point.

set cursor off

@  0,  0 clear
do while inkey() = 0
enddo

AppsQuit()

return .t.

*+
*+
*+    Procedure CHECKLOGOUT()
*+
*+    Called from ( timeout.prg  )   1 - function main()
*+
*+
*+
procedure CHECKLOGOUT()

if file( datadir() + "LOGOUT.NOW" )
   DEACTIVATE EVENT "CHECKLOGOUT"       // not needed any more.
   ACTIVATE EVENT "APPSQUIT"
   DEFINE BGEVENT ACTION COUNTDOWN() NAME COUNTDOWN         // show time left
endif

return

provedure COUNTDOWN()

@ maxrow(), 0 say ;         
           "The administrator forces you to logout. Save your work! Time left: "
@ row(), col() say ;         
       timeasstring( BGE_INTERVAL( "APPSQUIT" ) + BGE_TIMER( "COUNTDOWN" ) -
dseconds(), 4 )

//  dseconds() and BGE-functions are in bgevents.prg
return

*+
*+
*+    Procedure AppsQuit()
*+
*+    Called from ( timeout.prg  )   1 - procedure checkinactive()
*+                                   2 - function main()
*+
*+
*+
procedure AppsQuit

close all
quit

return

*+
*+
*+    Function datadir()
*+
*+    Called from ( timeout.prg  )   1 - procedure checklogout()
*+
*+
*+
function datadir()  // user your own

return ""

*+
*+
*+    Function TimeAsString()
*+
*+    Called from ( timeout.prg  )   1 - procedure checklogout()
*+
*+
*+
function TimeAsString( nSeconds )

return strzero( int( mod( nSeconds / 3600, 24 ) ), 2, 0 ) + ":" + ;
                strzero( int( mod( nSeconds / 60, 60 ) ), 2, 0 ) + ":" + ;
                strzero( int( mod( nSeconds, 60 ) ), 2, 0 )

*+ EOF: TIMEOUT.PRG
